class Quandl::Cassinatra::Base
module Search
  
  extend ActiveSupport::Concern

  included do

    include ScopeComposer::Model

    has_scope_composer

    scope.class_eval do
      
      def fetch_once
        @fetch_once ||= fetch
      end
      
      def fetch
        result = find(attributes[:id])
        result = references.imprint( result ) unless references.nil?
        result
      end
      
      def find(id)
        begin
          t1 = Time.now
          result = self.class.parent.where(attributes).find(id)
          Quandl::Logger.debug "#{self.class.name}.where(#{attributes.to_param if attributes.respond_to?(:to_param)}).find(#{id}) (#{t1.elapsed_ms})"
        rescue => error
          Quandl::Logger.error "#{self.class.name}.find #{error}"
        end
        result = self.class.parent.new(id: id) if result.nil?
        result
      end
      
      def references(*args)
        return @references if args.first.nil?
        @references = args.first
        attributes[:id]  = @references.id
        self
      end
      
      protected
      
      def format_trim_date(date, start_or_end)
        # parse
        date = parse_date(date)
        # format
        if date.is_a?(Date)
          date = date.send("#{start_or_end}_of_frequency", collapse) if collapse.present?
          date.jd
        else
          nil
        end
      end
    
      def parse_date( date )
        begin
          date = Date.jd(date.to_i) if date.kind_of?(String) && numeric?(date)
          date = Date.jd(date) if date.is_a?(Integer)
          date = Date.parse(date) if date.is_a?(String) && date =~ /^[0-9]{4}\-[0-9]{2}\-[0-9]{2}$/
          date
        rescue
          nil
        end
      end
    
      def numeric?(string)
        string = string.to_s
        if string =~ /^\d+$/
          true
        else
          Float(string).is_a?(Float) rescue false
        end
      end
      
    end

  end
  
  def reload
    # reset attributes
    @attributes = { 'id' => self.id }
    @metadata = {}
    # return self
    true
  end
  
  module ClassMethods
    def scope_names
      self.scope.scope_names
    end
  end
  
end
end