module Quandl
module Cassinatra
module Middleware

class ParseJSON < Faraday::Response::Middleware

  def on_complete(env)
    env[:body] = case env[:status]
    when 204
      parse('{}', env)
    else
      parse(env[:body], env)
    end
  end

  def parse(body, env)
    json = parse_json(body, env)
    errors = json.delete(:errors) || {}
    metadata = json.delete(:metadata) || {}
    # collect some response data
    metadata.merge!({
      status:                 env[:status],
      headers:                env[:response_headers],
      })
    # return object
    object = {
      :data => json,
      :errors => errors,
      :metadata => metadata
    }
    env[:status] = 200
    object
  end

  def parse_json(body = nil, env)
    body ||= '{}'
    json = begin
      Yajl.load(body, :symbolize_keys => true)
    rescue Yajl::ParseError
      nil
    end
    # invalid json body?
    if json.blank?
      # fallback to error message
      json = { 
        id: 1, 
        errors: { 
          parse_error:  "Quandl::Cassinatra::ParseJSON error. status: #{env[:status]}, body: #{body.inspect}"
        }
      }
    end
    json
  end

end

end
end
end